from typing import TypedDict, List, Literal
from enum import Enum


class QuranVersetData(TypedDict):
    """
    Représente les informations d'un verset du Coran.

    Attributes:
        position (int): Position globale du verset dans le Coran.
        text (str): Le texte du verset en translittération.
        position_ds_sourate (int): Position du verset dans la sourate.
        juz (int): Le numéro du Juz auquel appartient le verset.
        manzil (int): Le numéro du Manzil auquel appartient le verset.
        page (int): Le numéro de page où se trouve le verset.
        ruku (int): Le numéro du Ruku auquel appartient le verset.
        hizbQuarter (int): Le quart de Hizb auquel appartient le verset.
        sajda (bool): Indique si le verset contient une prosternation (Sajda).
        text_arabe (str): Le texte du verset en arabe.
    """

    position: int
    text: str
    position_ds_sourate: int
    juz: int
    manzil: int
    page: int
    ruku: int
    hizbQuarter: int
    sajda: bool
    text_arabe: str


class QuranSurahsData(TypedDict):
    """
    Représente les informations d'une sourate du Coran.

    Attributes:
        position (int): La position de la sourate dans le Coran.
        nom (str): Le nom de la sourate en arabe.
        nom_phonetique (str): La translittération du nom de la sourate.
        englishNameTranslation (str): La traduction anglaise du nom de la sourate.
        revelation (str): Le type de révélation (Medinois ou Mecquoise).
        versets (List[QuranVersetData]): La liste des versets de la sourate.
        nom_sourate (str): Le nom de la sourate en français.
    """

    position: int
    nom: str
    nom_phonetique: str
    englishNameTranslation: str
    revelation: str
    versets: List[QuranVersetData]
    nom_sourate: str


class QuranData(TypedDict):
    """
    Représente les données du Coran avec toutes les sourates.

    Attributes:
        sourates (List[QuranSouratetData]): Liste des sourates avec leurs informations.
    """

    sourates: List[QuranSurahsData]


class RevelationType(Enum):
    """
    Enumération des types de révélations des sourates.

    Values:
        MEDINOIS (str): Révélée à Médine.
        MECQUOISE (str): Révélée à La Mecque.
    """

    MEDINOIS = "Medinois"
    MECQUOISE = "Mecquoise"


FilterVersets = Literal["position_ds_sourate"]


class FilterVersetsDict(TypedDict):
    """
    Représente un filtre pour les versets basé sur la position dans une sourate.

    Attributes:
        position_ds_sourate (int): Position d'un verset dans une sourate.
    """

    position_ds_sourate: int


TypeFilterSearch = Literal["arabic", "fr", "phonetic", "number"]
TypeFilterVerset = Literal["arabic", "fr"]


TypeNamesFr = Literal[
    "Prologue",
    "La vache",
    "La famille d'Imran",
    "Les femmes",
    "La table servie",
    "Les bestiaux",
    "Al-Araf",
    "Le butin",
    "Le repentir",
    "Jonas",
    "Hud",
    "Joseph",
    "Le tonnerre",
    "Abraham",
    "Al-Hijr",
    "Les abeilles",
    "Le voyage nocturne",
    "La caverne",
    "Marie",
    "Ta-Ha",
    "Les prophètes",
    "Le pélerinage",
    "Les croyants",
    "La lumière",
    "Le discernement",
    "Les poètes",
    "Les fourmis",
    "Le récit",
    "L'araignée",
    "Les romains",
    "Luqman",
    "La prosternation",
    "Les coalisés",
    "Saba",
    "Le Créateur",
    "Ya-Sin",
    "Les rangés",
    "Sad",
    "Les groupes",
    "Le pardonneur",
    "Les versets détaillés",
    "La consultation",
    "L'ornement",
    "La fumée",
    "L'agenouillée",
    "Al-Ahqaf",
    "Muhammad",
    "La victoire éclatante",
    "Les appartements",
    "Qaf",
    "Qui éparpillent",
    "At-Tur",
    "L'étoile",
    "La lune",
    "Le Tout Miséricordieux",
    "L'événement",
    "Le fer",
    "La discussion",
    "L'exode",
    "L'éprouvée",
    "Le rang",
    "Le vendredi",
    "Les hypocrites",
    "La grande perte",
    "Le divorce",
    "L'interdiction",
    "La royauté",
    "La plume",
    "Celle qui montre la vérité",
    "Les voies d'ascension",
    "Noé",
    "Les djinns",
    "L'enveloppé",
    "Le revêtu d'un manteau",
    "La résurrection",
    "L'homme",
    "Les envoyés",
    "La nouvelle",
    "Les anges qui arrachent les âmes",
    "Il s'est renfrogné",
    "L'obscurcissement",
    "La rupture",
    "Les fraudeurs",
    "La déchirure",
    "Les constellations",
    "L'astre nocturne",
    "Le Très-Haut",
    "L'enveloppante",
    "L'aube",
    "La cité",
    "Le soleil",
    "La nuit",
    "Le jour montant",
    "L'ouverture",
    "Le figuier",
    "L'adhérence",
    "La Destinée",
    "La preuve",
    "La secousse",
    "Les coursiers",
    "Le fracas",
    "La course aux richesses",
    "Le temps",
    "Les calomniateurs",
    "L'éléphant",
    "Qoraïsh",
    "L'ustensile",
    "L'abondance",
    "Les infidèles",
    "Les secours",
    "Les fibres",
    "Le monothéisme pur",
    "L'aube naissante",
    "Les hommes",
]

TypeNamesPhonetic = Literal[
    "Al-Faatiha",
    "Al-Baqara",
    "Aal-i-Imraan",
    "An-Nisaa",
    "Al-Maaida",
    "Al-An'aam",
    "Al-A'raaf",
    "Al-Anfaal",
    "At-Tawba",
    "Yunus",
    "Hud",
    "Yusuf",
    "Ar-Ra'd",
    "Ibrahim",
    "Al-Hijr",
    "An-Nahl",
    "Al-Israa",
    "Al-Kahf",
    "Maryam",
    "Taa-Haa",
    "Al-Anbiyaa",
    "Al-Hajj",
    "Al-Muminoon",
    "An-Noor",
    "Al-Furqaan",
    "Ash-Shu'araa",
    "An-Naml",
    "Al-Qasas",
    "Al-Ankaboot",
    "Ar-Room",
    "Luqman",
    "As-Sajda",
    "Al-Ahzaab",
    "Saba",
    "Faatir",
    "Yaseen",
    "As-Saaffaat",
    "Saad",
    "Az-Zumar",
    "Ghafir",
    "Fussilat",
    "Ash-Shura",
    "Az-Zukhruf",
    "Ad-Dukhaan",
    "Al-Jaathiya",
    "Al-Ahqaf",
    "Muhammad",
    "Al-Fath",
    "Al-Hujuraat",
    "Qaaf",
    "Adh-Dhaariyat",
    "At-Tur",
    "An-Najm",
    "Al-Qamar",
    "Ar-Rahmaan",
    "Al-Waaqia",
    "Al-Hadid",
    "Al-Mujaadila",
    "Al-Hashr",
    "Al-Mumtahana",
    "As-Saff",
    "Al-Jumu'a",
    "Al-Munaafiqoon",
    "At-Taghaabun",
    "At-Talaaq",
    "At-Tahrim",
    "Al-Mulk",
    "Al-Qalam",
    "Al-Haaqqa",
    "Al-Ma'aarij",
    "Nooh",
    "Al-Jinn",
    "Al-Muzzammil",
    "Al-Muddaththir",
    "Al-Qiyaama",
    "Al-Insaan",
    "Al-Mursalaat",
    "An-Naba",
    "An-Naazi'aat",
    "Abasa",
    "At-Takwir",
    "Al-Infitaar",
    "Al-Mutaffifin",
    "Al-Inshiqaaq",
    "Al-Burooj",
    "At-Taariq",
    "Al-A'laa",
    "Al-Ghaashiya",
    "Al-Fajr",
    "Al-Balad",
    "Ash-Shams",
    "Al-Lail",
    "Ad-Dhuhaa",
    "Ash-Sharh",
    "At-Tin",
    "Al-Alaq",
    "Al-Qadr",
    "Al-Bayyina",
    "Az-Zalzala",
    "Al-Aadiyaat",
    "Al-Qaari'a",
    "At-Takaathur",
    "Al-Asr",
    "Al-Humaza",
    "Al-Fil",
    "Quraish",
    "Al-Maa'un",
    "Al-Kawthar",
    "Al-Kaafiroon",
    "An-Nasr",
    "Al-Masad",
    "Al-Ikhlaas",
    "Al-Falaq",
    "An-Naas",
]

TypeNamesFrLower = Literal[
    "prologue",
    "la vache",
    "la famille d'imran",
    "les femmes",
    "la table servie",
    "les bestiaux",
    "al-araf",
    "le butin",
    "le repentir",
    "jonas",
    "hud",
    "joseph",
    "le tonnerre",
    "abraham",
    "al-hijr",
    "les abeilles",
    "le voyage nocturne",
    "la caverne",
    "marie",
    "ta-ha",
    "les prophètes",
    "le pélerinage",
    "les croyants",
    "la lumière",
    "le discernement",
    "les poètes",
    "les fourmis",
    "le récit",
    "l'araignée",
    "les romains",
    "luqman",
    "la prosternation",
    "les coalisés",
    "saba",
    "le créateur",
    "ya-sin",
    "les rangés",
    "sad",
    "les groupes",
    "le pardonneur",
    "les versets détaillés",
    "la consultation",
    "l'ornement",
    "la fumée",
    "l'agenouillée",
    "al-ahqaf",
    "muhammad",
    "la victoire éclatante",
    "les appartements",
    "qaf",
    "qui éparpillent",
    "at-tur",
    "l'étoile",
    "la lune",
    "le tout miséricordieux",
    "l'événement",
    "le fer",
    "la discussion",
    "l'exode",
    "l'éprouvée",
    "le rang",
    "le vendredi",
    "les hypocrites",
    "la grande perte",
    "le divorce",
    "l'interdiction",
    "la royauté",
    "la plume",
    "celle qui montre la vérité",
    "les voies d'ascension",
    "noé",
    "les djinns",
    "l'enveloppé",
    "le revêtu d'un manteau",
    "la résurrection",
    "l'homme",
    "les envoyés",
    "la nouvelle",
    "les anges qui arrachent les âmes",
    "il s'est renfrogné",
    "l'obscurcissement",
    "la rupture",
    "les fraudeurs",
    "la déchirure",
    "les constellations",
    "l'astre nocturne",
    "le très-haut",
    "l'enveloppante",
    "l'aube",
    "la cité",
    "le soleil",
    "la nuit",
    "le jour montant",
    "l'ouverture",
    "le figuier",
    "l'adhérence",
    "la destinée",
    "la preuve",
    "la secousse",
    "les coursiers",
    "le fracas",
    "la course aux richesses",
    "le temps",
    "les calomniateurs",
    "l'éléphant",
    "qoraïsh",
    "l'ustensile",
    "l'abondance",
    "les infidèles",
    "les secours",
    "les fibres",
    "le monothéisme pur",
    "l'aube naissante",
    "les hommes",
]

TypeNamesPhoneticLower = Literal[
    "al-faatiha",
    "al-baqara",
    "aal-i-imraan",
    "an-nisaa",
    "al-maaida",
    "al-an'aam",
    "al-a'raaf",
    "al-anfaal",
    "at-tawba",
    "yunus",
    "hud",
    "yusuf",
    "ar-ra'd",
    "ibrahim",
    "al-hijr",
    "an-nahl",
    "al-israa",
    "al-kahf",
    "maryam",
    "taa-haa",
    "al-anbiyaa",
    "al-hajj",
    "al-muminoon",
    "an-noor",
    "al-furqaan",
    "ash-shu'araa",
    "an-naml",
    "al-qasas",
    "al-ankaboot",
    "ar-room",
    "luqman",
    "as-sajda",
    "al-ahzaab",
    "saba",
    "faatir",
    "yaseen",
    "as-saaffaat",
    "saad",
    "az-zumar",
    "ghafir",
    "fussilat",
    "ash-shura",
    "az-zukhruf",
    "ad-dukhaan",
    "al-jaathiya",
    "al-ahqaf",
    "muhammad",
    "al-fath",
    "al-hujuraat",
    "qaaf",
    "adh-dhaariyat",
    "at-tur",
    "an-najm",
    "al-qamar",
    "ar-rahmaan",
    "al-waaqia",
    "al-hadid",
    "al-mujaadila",
    "al-hashr",
    "al-mumtahana",
    "as-saff",
    "al-jumu'a",
    "al-munaafiqoon",
    "at-taghaabun",
    "at-talaaq",
    "at-tahrim",
    "al-mulk",
    "al-qalam",
    "al-haaqqa",
    "al-ma'aarij",
    "nooh",
    "al-jinn",
    "al-muzzammil",
    "al-muddaththir",
    "al-qiyaama",
    "al-insaan",
    "al-mursalaat",
    "an-naba",
    "an-naazi'aat",
    "abasa",
    "at-takwir",
    "al-infitaar",
    "al-mutaffifin",
    "al-inshiqaaq",
    "al-burooj",
    "at-taariq",
    "al-a'laa",
    "al-ghaashiya",
    "al-fajr",
    "al-balad",
    "ash-shams",
    "al-lail",
    "ad-dhuhaa",
    "ash-sharh",
    "at-tin",
    "al-alaq",
    "al-qadr",
    "al-bayyina",
    "az-zalzala",
    "al-aadiyaat",
    "al-qaari'a",
    "at-takaathur",
    "al-asr",
    "al-humaza",
    "al-fil",
    "quraish",
    "al-maa'un",
    "al-kawthar",
    "al-kaafiroon",
    "an-nasr",
    "al-masad",
    "al-ikhlaas",
    "al-falaq",
    "an-naas",
]
